@@ -3,7 +3,7 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapobject.py,v 1.149 2015/10/24 15:46:12 stroeder Exp $
+\$Id: ldapobject.py,v 1.159 2017/04/24 08:25:16 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -206,8 +206,7 @@ class SimpleLDAPObject:
     return self.add_ext(dn,modlist,None,None)
 
   def add_s(self,dn,modlist):
-    msgid = self.add(dn,modlist)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    return self.add_ext_s(dn,modlist,None,None)
 
   def simple_bind(self,who='',cred='',serverctrls=None,clientctrls=None):
     """
@@ -398,8 +397,7 @@ class SimpleLDAPObject:
     return self.modify_ext(dn,modlist,None,None)
 
   def modify_s(self,dn,modlist):
-    msgid = self.modify(dn,modlist)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    return self.modify_ext_s(dn,modlist,None,None)
 
   def modrdn(self,dn,newrdn,delold=1):
     """
@@ -623,13 +621,25 @@ class SimpleLDAPObject:
         The unbind and unbind_s methods are identical, and are
         synchronous in nature
     """
-    return self._ldap_call(self._l.unbind_ext,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
+    res = self._ldap_call(self._l.unbind_ext,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
+    try:
+      del self._l
+    except AttributeError:
+      pass
+    return res
 
   def unbind_ext_s(self,serverctrls=None,clientctrls=None):
     msgid = self.unbind_ext(serverctrls,clientctrls)
     if msgid!=None:
-      resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
-      return resp_type, resp_data, resp_msgid, resp_ctrls
+      result = self.result3(msgid,all=1,timeout=self.timeout)
+    else:
+      result = None
+    if __debug__ and self._trace_level>=1:
+      try:
+        self._trace_file.flush()
+      except AttributeError:
+        pass
+    return result
 
   def unbind(self):
     return self.unbind_ext(None,None)
@@ -737,6 +747,26 @@ class SimpleLDAPObject:
       raise NO_UNIQUE_ENTRY('No or non-unique search result for %s' % (repr(filterstr)))
     return r[0]
 
+  def read_rootdse_s(self, filterstr='(objectClass=*)', attrlist=None):
+    """
+    convenience wrapper around read_s() for reading rootDSE
+    """
+    ldap_rootdse = self.read_s(
+      '',
+      filterstr=filterstr,
+      attrlist=attrlist or ['*', '+'],
+    )
+    return ldap_rootdse  # read_rootdse_s()
+
+  def get_naming_contexts(self):
+    """
+    returns all attribute values of namingContexts in rootDSE
+    if namingContexts is not present (not readable) then empty list is returned
+    """
+    return self.read_rootdse_s(
+      attrlist=['namingContexts']
+    ).get('namingContexts', [])
+
 
 class NonblockingLDAPObject(SimpleLDAPObject):
 
@@ -848,6 +878,9 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     for k,v in self._options:
       SimpleLDAPObject.set_option(self,k,v)
 
+  def passwd_s(self,*args,**kwargs):
+    return self._apply_method_s(SimpleLDAPObject.passwd_s,*args,**kwargs)
+
   def reconnect(self,uri,retry_max=1,retry_delay=60.0):
     # Drop and clean up old connection completely
     # Reconnect
@@ -881,7 +914,6 @@ class ReconnectLDAPObject(SimpleLDAPObject):
             self._trace_file.write('=> delay %s...\n' % (retry_delay))
           time.sleep(retry_delay)
           SimpleLDAPObject.unbind_s(self)
-          del self._l
         else:
           if __debug__ and self._trace_level>=1:
             self._trace_file.write('*** %s reconnect to %s successful => repeat last operation\n' % (
@@ -900,7 +932,6 @@ class ReconnectLDAPObject(SimpleLDAPObject):
       return func(self,*args,**kwargs)
     except ldap.SERVER_DOWN:
       SimpleLDAPObject.unbind_s(self)
-      del self._l
       # Try to reconnect
       self.reconnect(self._uri,retry_max=self._retry_max,retry_delay=self._retry_delay)
       # Re-try last operation
@@ -944,8 +975,8 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   def cancel_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.cancel_s,*args,**kwargs)
 
-  def compare_s(self,*args,**kwargs):
-    return self._apply_method_s(SimpleLDAPObject.compare_s,*args,**kwargs)
+  def compare_ext_s(self,*args,**kwargs):
+    return self._apply_method_s(SimpleLDAPObject.compare_ext_s,*args,**kwargs)
 
   def delete_ext_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.delete_ext_s,*args,**kwargs)
